#
# Copyright (c) Daniel Sheffield 2024
# All rights reserved
#
# THIS SOFTWARE IS PROVIDED AS IS WITHOUT WARRANTY
from io import BytesIO
import time
from bottle import (
    Bottle,
    default_app,
    route, request, response,
    redirect, abort,
    template, static_file,
    HTTPResponse,
)
from linkpreview import link_preview

from .validate import CLIP_SIZE_LIMIT, get_file_mimetype, get_file_size, get_filename, validate_file
from .hash_util import B32REGEX, normalize_base32, blake, bytes_to_base32
from .save import save_upload
from .qr import get_qr_code
from json import dumps, load

from sqlite3 import connect, Row

SCHEME = "https://"
HOST = ""
DOMAIN = "shandan.one"
PORT = ""
LOCATION = SCHEME + (f"{HOST}." if HOST else "") + DOMAIN + (f":{PORT}" if PORT else "")

@route('/preview', method=['GET'])
def get_preview():
    link = request.params.link
    if not link:
        return dumps(None)
    try:
        page = link_preview(link, parser="lxml")
        return dumps({
            'title': page.title,
            'img': page.absolute_image,
            'domain': page.site_name,
            'link': link,
        })
    except:
        return dumps(None)

@route('/hash', method=['POST'])
def get_hash():
    data = dict(map(
        lambda x: (x[0], x[1].encode('utf-8')),
        load(request.body).items()
    ))
    _bytes = blake(**data)
    return bytes_to_base32(_bytes)

@route('/normalize', method=['GET'])
def normalize():
    _hash = request.params.hash
    response.content_type = 'application/json'
    return dumps({
        'i': _hash,
        'o': normalize_base32(_hash) if _hash else None,
    })

@route('/qr', method=['POST'])
def get_qr():
    data = load(request.body)
    return get_qr_code(**data).decode('utf-8')

@route('/static/<filename:path>')
def send_static(filename):
    return static_file(filename, root='rest/static')

@route('/upload/open')
def _get_clip():
    return template('form-open', tool='upload', action=f'/upload', method='get')

@route('/<route:re:(clip|goto)>/open')
def _get_clip(route):
    return redirect(f'/{route}/open.sql')

@route('/<route:re:(clip|goto)>', method=['GET', 'POST'])
def clip(route):
    return redirect(f'/{route}.sql')

@route('/<route:re:(clip|goto)>/<filename:path>', method='GET')
def get_clip(route, filename):
    return redirect(f'/{route}.sql?hash={filename}&go=true')

@route(f'/upload/<hash:re:{B32REGEX}{{1,5}}>', method='GET')
def get_upload(hash, db=None):
    hash = hash and normalize_base32(hash)
    con = connect('util.db')
    try:
        ret: Row = con.cursor().execute(f"""
SELECT content
FROM clip
WHERE hash = '{hash}'
LIMIT 1;
""").fetchall()[0][0].encode('utf-8')
    finally:
        con.close()
   
    download = True
    mimetype = True
    if request.params.download == "false":
        download = False
    mimetype = request.params.mimetype or None
    headers = {}
    headers['Content-Length'] = len(ret)
    fname = 'noddy'
    headers['Content-Disposition'] = 'attachment; filename="%s"' % fname
    headers['Content-Encoding'] = 'application/octet-stream'
    # if mimetype == 'auto':
    #     mimetype, encoding = mimetypes.guess_type(filename)
    #     if encoding: headers['Content-Encoding'] = encoding

    # if mimetype:
    #     if mimetype[:5] == 'text/' and charset and 'charset' not in mimetype:
    #         mimetype += '; charset=%s' % charset
    #     headers['Content-Type'] = mimetype
    #lm = time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime(created))
    #headers['Last-Modified'] = lm
    return HTTPResponse(ret, **headers)

@route('/upload', method=['GET', 'POST'])
def upload():

    if request.method == 'GET':
        _hash = request.params.hash
        mimetype = None
        if _hash:
            _hash = normalize_base32(_hash)
            size = get_file_size(_hash)
            name = get_filename(_hash)
            mimetype = get_file_mimetype(name)
            validate_file(_hash, root='rest/static/files', download=True, mimetype=mimetype)
            if mimetype and mimetype is not True and mimetype.startswith('text'):
                mimetype = None if size and size > CLIP_SIZE_LIMIT else mimetype
        
        link = f'{LOCATION}/upload/{_hash}' if _hash else f'{LOCATION}/upload'
        response.content_type = 'text/html; charset=utf-8'
        disabled = True if _hash else False
        form = template('form-upload', action='/upload', method='post', disabled=disabled)
        
        return template(
            'upload',
            form=form,
            qr=f'{LOCATION}/upload/{_hash}.qr' if _hash else f'{LOCATION}/static/upload/qr.svg',
            link=link,
            mimetype=mimetype,
            disabled=disabled
        )

    if request.method == 'POST':
        if 'paste' not in request.files:
            return abort(400, "Parameter 'paste' must be specified")

        upload = request.files['paste']
        if isinstance(upload.file, BytesIO):
            if len(upload.file.read()) == 0:
                return abort(400, "File is empty")

        _b32 = save_upload(upload.raw_filename, LOCATION, upload.file, root='rest/static/files')
        return redirect(f'/upload?hash={_b32}')


# @route('/upload/<filename:path>', method='GET')
# def get_upload(filename):
#     ext = 'file'
#     if filename and filename.endswith('.qr'):
#         filename, ext = filename.split('.', 1)
    
#     filename = filename and normalize_base32(filename)
#     path = f'{filename}/{filename}.{ext}'
    
#     if ext == 'qr':
#         return static_file(path, root='rest/static/files', mimetype='image/svg+xml')
    
#     download = True
#     mimetype = True
#     if request.params.download == "false":
#         download = False
#     mimetype = request.params.mimetype or None

#     return validate_file(filename, root='rest/static/files', download=download, mimetype=mimetype)


@route('/<any>/', method='GET')
def redirect_trailing_slash(any): return redirect(f'/{any}')
